package lsedit;

import java.awt.Cursor;
import java.awt.Event;
import java.awt.event.MouseEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JPopupMenu;

// The Edit and View ModeHandler are toggled by typing 'V'
// However this only occurs if we are using the landscapeEditorCore 

public class EditModeHandler extends ViewModeHandler {

	protected JPopupMenu buildEntityPopup()
	{
		JPopupMenu m;
		JMenu	   m1;
		
		m = new JPopupMenu("Entity options");

		m1 = new JMenu("Navigate");
		Do.navigateEntityMenu(m1, m_ls);
		m.add(m1);
		m1 = new JMenu("Query");
		Do.queryMenuItem(m1, m_ls);
		m.add(m1);

		m1 = new JMenu("Elide");
		Do.hideMenuItem(m1, m_ls);
		m.add(m1);

		m1 = new JMenu("Draw");
		Do.drawMenuItem(m1, m_ls);
		m1.addSeparator();
		Do.editAttributesMenuItem(m1, m_ls);
		m1.addSeparator();
		Do.fontMenuItem(m1, m_ls);
		m.addSeparator();
		m.add(m1);

		m1 = new JMenu("Move");
		Do.groupAllMenuItem(m1, m_ls);
		m.addSeparator();
		Do.scaleMenuItem(m1, m_ls);
		m.add(m1);

		return(m);
	}

	protected JPopupMenu buildRelationPopup()
	{
		JPopupMenu m;
		JMenu	   m1;
		
		m = new JPopupMenu("Edge options");
		Do.navigateEdgeMenu(m, m_ls);
		Do.deleteEdgeMenuItem(m, m_ls);
		Do.editRelationMenuItem(m, m_ls);
		return(m);
	}

	// -----------------
	// Protected methods
	// -----------------

	// Things can't do in ViewModeHandler should go here

	protected boolean handleCommands(int key, int modifiers, Object object) 
	{
		boolean	handled = true;

//		System.out.println("EditModeHandler:handleCommands(" + key + ", " + ", " + modifiers + ", " + object +")");


		if (modifiers != 0) {
			handled = false;
		} else {
			switch(key) {
			case Do.DELETE:
				m_ls.doDelete(object);
				return true;
			case Do.INCREASE_LABEL_FONT:
				m_ls.deltaFont(1);
				break;
			case Do.DECREASE_LABEL_FONT: 
				m_ls.deltaFont(-1);
				break;
			case Do.NEW_EDGE:
			{
				if (m_newEdgeModeHandler == null) {
					m_newEdgeModeHandler = new NewEdgeModeHandler(this);
				}
				m_newEdgeModeHandler.activate(object);
				return true;
			}
			case Do.NEW_ENTITY:
			{
				Diagram		   diagram = m_ls.getDiagram();
				EntityInstance e       = diagram.targetEntity(object);
				if (e == null) {
					return false;
				}				
				m_ls.newEntity(e, null);
				break;
			}

			case Do.EDIT_ENTITY_CLASS:
			{
				Diagram	diagram = m_ls.getDiagram();
				if (diagram == null) {
					return false;
				}
				LandscapeObject landscapeObject = diagram.targetEntityRelation(object);
				if (landscapeObject == null) {
					return false;
				}
				if (landscapeObject instanceof EntityInstance) {
					EntityInstance e = (EntityInstance) landscapeObject;
					// edit class attributes
					EditAttribute.Create(m_ls, e.getEntityClass());
				} else {
					RelationInstance ri = (RelationInstance) landscapeObject;
					// edit class attributes
					EditAttribute.Create(m_ls, ri.getRelationClass());
				}
				break;
			}
			case Do.EDIT_ATTRIBUTES:
			{
				Diagram			diagram         = m_ls.getDiagram();
				if (diagram == null) {
					return false;
				}
				LandscapeObject landscapeObject = diagram.targetEntityRelation(object);
				if (landscapeObject == null) {
					return false;
				}
				if (landscapeObject instanceof EntityInstance) {
					EntityInstance e = (EntityInstance) landscapeObject;
					// edit attributes
					EditAttribute.Create(m_ls, e);
				} else {
					RelationInstance ri = (RelationInstance) landscapeObject;
					// edit attributes
					EditAttribute.Create(m_ls, ri);
				}
				break;
			}
			case Do.DELETE_CONTAINER:
			{
				Diagram		   diagram = m_ls.getDiagram();
				EntityInstance e       = diagram.targetEntity(object);

				if (e == null) {
					return false;
				}
				if (e == diagram.getRootInstance()) {
					m_ls.error("Can't delete root instance");
					return false;
				}
				m_ls.deleteContainer(e);
				break;
			}
			case Do.MOVE_REDBOXES:
				m_ls.moveRedBoxes(object);
				break;
			default:
				handled = false;
		}	}
		if (handled) {
			m_ls.redrawDg();
		} else {
			handled = super.handleCommands(key, modifiers, object);
		}
		return handled;
	}

	// --------------
	// Public methods
	// --------------

	public EditModeHandler(LandscapeViewerCore ls) 
	{
		super(ls);
	}
}
